#ifndef __include_obj_h__
#define __include_obj_h__

#ifdef RUNDEBUG
typedef void * POOL;
#define ASSERT(x)	assert(x)
#include <assert.h>
#define palloc(p,s)	malloc(s)
#define pool_new(p)	(void *)(1)
#define pstrdup(p,s)	strdup(s)
#define pool_destroy(p) do{} while(0)
#include <string.h>
#endif

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>


typedef unsigned int HANDLE;
typedef unsigned short obj_type_h;
#define HANDLE_INVALID	0xFFFFFFFF
#define obj_type_invalid	0xFFFF
#define obj_type_any		obj_type_invalid
#define obj_type_none		obj_type_invalid

/* Error codes */
#define obj_err_ok		0
#define obj_err_invalid_name	-1
#define obj_err_invalid_type	-2
#define obj_err_invalid_handle	-3
#define obj_err_invalid_ptr	-4
#define obj_err_not_type	-5
#define obj_err_invalid_obj	-6
#define obj_err_no_hash		-7
#define obj_err_mem		-8
#define obj_err_already		-9
#define obj_err_freed		-10
#define obj_err_grabbed		-11

int object_get_errno();

/* This function is used to register a new object type.
 * The name of the object type is specified, and the resultant 
 * obj_type_h handle is returned in ans.
 * 
 * Error Codes: This function can return one of these error codes
 * obj_err_ok:			No problems.
 * obj_err_invalid_name:	The name specified isn't valid, or is
 * 				NULL.
 * obj_err_invalid_ptr:		The pointer to ans provided isn't valid.
 */
int object_register_type(obj_type_h *ans, const char *name);

/* This function creates a new object of the type specified.
 * It uses the pool specified for the objects parent pool. 
 * It allocates an object of the specified size. A pointer to the
 * memory is returned in ptr.
 *
 * Error codes: This function can set one of these error codes
 * obj_err_ok:			No problems
 * obj_err_invalid_ptr:		The pointer is invalid.
 * obj_err_invalid_type:	The type specified is invalid
 */
void *object_new(obj_type_h type, size_t size);

/* This function returns the HANDLE of a given object. 
 * Error codes:
 * This function returns either a valid handle, or HANDLE_INVALID
 * which indicates that obj is not an actual object (though the SIGSEGV
 * may give that away before it returns).
 */
HANDLE object_get_handle(void *obj);

/* This function returns a pointer to the structure represented by this
 * handle. Pass the type you were expecting. A pointer to the resultant
 * structure is set in ptr. If you pass obj_type_any, then no type
 * checking will be done(!).
 * 
 * Error codes: This function sets one of these error codes
 * obj_err_ok:			No problems
 * obj_err_invalid_handle:	The handle is invalid.
 * obj_err_invalid_type:	The expecting type is invalid
 * obj_err_not_type:		The handle is not a handle to the
 * 				specified type.
 * 
 */
void *object_get_obj(HANDLE h, obj_type_h expecting);

/* object_grab stops the object being free'd until the object is then
 * released. Object_grab is cumulative, so if you grab 3 times, you'll
 * need to release 3 times to allow a free to go ahead.
 */
int object_grab(HANDLE h);

/* object_release releases an object after grabbing it. */
int object_release(HANDLE h);

/* object_attach
 * Description:
 * 	This attaches a specified child object to the specified parent
 * 	object. This attachment allows the child object to be found
 * 	using the parent object (see object_get_child()).
 * 	When the parent object is freed, the child object is destroyed
 * 	aswell.
 * 	You can not object_grab() a child object, and any object
 * 	that is grabbed, can not become a child object.
 * 	You can not register two objects are child objects of a parent
 * 	if they are both of the same type. Each child must be of
 * 	a different type.
 *
 * Returns: obj_err_ok on Success, or the error code on failure.
 * 
 * Error Codes:
 * obj_err_grabbed: The child has been grabbed, and therefore can not
 * 	become a child object.
 * obj_err_invalid_obj: One of the objects specified is invalid.
 * obj_err_existing: There is already an existing child object of that
 * 	type.
 */
int object_attach(HANDLE parent, HANDLE child);

/* object_get_child(parent, type)
 * Description:
 * 	Retrieves the child object of a parent of a specified type.
 * 	If there is no such child of that type, obj_err_not_found
 * 	is set.
 * Returns: The HANDLE of the child object, or HANDLE_INVALID in
 * 	the event of error. (Use object_get_errno() to get error
 * 	code).
 *
 * Errors:
 * 	obj_err_not_found: No such child was found.
 * 	obj_err_not_type: The type specified isn't a real type.
 * 	obj_err_invalid_obj: The parent object is invalid.
 */
HANDLE object_get_child(HANDLE parent, obj_type_h type);

/* object_count_children(parent, count)
 * Description:
 * 	Sets count to contain the number of children in a parent object.
 * Returns: obj_err_ok, or the error code in the event of an error.
 * Errors:
 * 	object_err_invalid_obj: The parent isn't a valid object.
 */
int object_count_children(HANDLE parent, int *count);

/* object_detach(parent,child)
 * Description:
 * 	Attempts to detach the child object from the parent object.
 */
int object_detach(HANDLE parent, HANDLE child);

/* Things we can do on an object 
 * Generally you shouldn't be doing these things unless you created
 * the object.
 */
/* object_free requests an object be freed.
 * If the object is currently grabbed, then object will be freed
 * when the grab is released.
 * If the object is grabbed (eg returns obj_err_grabbed), do not
 * call object_free again, as it will be freed when it is released
 * This function returns on of the following codes, however in all
 * cases, it is an error to call object_free again.
 * obj_err_ok:			Object freed ok
 * obj_err_grabbed:		Object not freed yet, still grabbed
 * obj_err_already:		Object already freed(!).
 */
int object_free(void *obj);

/* Returns the pool used to allocate this object */
POOL object_get_pool(void *obj);

/* Returns the type of the given object, or obj_type_invalid in the
 * event of an error
 */
obj_type_h object_get_type(void *obj);
/* Returns a const character string representing the name of the type
 * of the specified object. Returns NULL if the object isn't an
 * object.
 */
const char *object_get_type_str(void *obj);

/* Returns the number of grabs done on an object. */
int object_get_grab_count(void *obj);


#endif
